<?php
/**
* NOTICE OF LICENSE
*
* This file is licenced under the Software License Agreement.
* With the purchase or the installation of the software in your application
* you accept the licence agreement.
*
* Do not edit or add to this file if you wish to upgrade Envia to newer
* versions in the future. If you wish to customize Envia for your
* needs please refer to http://www.envia.com for more information.
*
*  @author    Envia.com - Shipping Platform <marcelo@envia.com>
*  @copyright 2017-2018 Tendencys Innovations S.A. de C.V.
*  @license   LICENSE.txt
*/

if ( !defined('_PS_VERSION_') ) {
    exit;
}

class EnviaPrestashopModule extends Module
{
    public function __construct()
    {
        $this->name = 'enviaprestashopmodule';
        $this->tab = 'shipping_logistics';
        $this->version = '3.0.0';
        $this->author = 'Tendencys Innovations S.A. de C.V.';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->bootstrap = true;
        $this->module_key = '6a497d1c8866d111b3864c03a99b2e0e';
        $this->dbPrefix = _DB_PREFIX_;

        parent::__construct();

        $this->displayName = $this->l('Envia.com - Shipping Platform');
        $this->description = $this->l('Connect your Store with our platform to send your packages more easier.');
        $this->confirmUnistall = $this->l('Are you sure to delete this module?');

        $this->enviaHostname = "https://ship.envia.com";
        $this->url = _PS_BASE_URL_.__PS_BASE_URI__;
        $this->adminUrl = $this->url.basename(_PS_ADMIN_DIR_);

        if ( !Configuration::get('ENVIA_MODULE_NAME') ) {
            $this->warning = $this->l('No name provided');
        }
    }


    public function install()
    {
        if ( Shop::isFeatureActive() ) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        return parent::install() &&
        $this->registerHook('backOfficeHeader') &&
        $this->registerHook('adminOrder') &&
        $this->registerHook('header') &&
        Configuration::updateValue('ENVIA_MODULE_NAME', $this->displayName);
    }

    public function uninstall()
    {
        if ( Shop::isFeatureActive() ) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        return parent::uninstall() &&
        $this->unregisterHook('backOfficeHeader') &&
        $this->unregisterHook('adminOrder') &&
        $this->unregisterHook('header') &&
        $this->uninstallApp();
    }

    public function hookdisplayBackOfficeHeader($params)
    {
        $token = $this->createWebservice();
        $smarty = array(
            'url' => $this->url,
            'token' => $token,
            "enviaHostname" => $this->enviaHostname,
            "js" => "{$this->_path}views/js/prestashop_module.js",
            "css" => "{$this->_path}views/css/prestashop_module.min.css"
        );
        $this->context->smarty->assign($smarty);
        return $this->display(__FILE__, 'header.tpl');
    }

    public function hookdisplayAdminOrder($params)
    {
        $id_order = (int)$params['id_order'];
        $employee = $params['smarty']->{'tpl_vars'}['employee']->{'value'};
        $shop = $params['smarty']->{'tpl_vars'}['shop']->{'value'};
        $email = $employee->{'email'};
        $shopName = $shop->{'name'};
        $token = $this->createWebservice();

        $smarty = array(
            'id_order' => $id_order,
            'url' => $this->url,
            'token' => $token,
            "enviaHostname" => $this->enviaHostname
        );

        $this->context->smarty->assign($smarty);
        return $this->display(__FILE__, 'adminorder.tpl');
    }

    protected function createWebservice()
    {
        $webservice = false;
        $webserviceValues = array("PS_WEBSERVICE", "PS_WEBSERVICE_CGI_HOST");
        foreach ( $webserviceValues as $config ){
            $ws = Db::getInstance()->executeS("select * from {$this->dbPrefix}configuration where name = '{$config}'");
            if(count($ws) > 0){
                if ( $ws[0]["value"] == 1 ) {
                    $webservice = true;
                } else {
                    $webservice = Db::getInstance()->update("configuration", array("value" => 1), "name = '{$ws[0]["name"]}'");
                }
            }else{
                $insert = Db::getInstance()->insert("configuration", array("name" => $config, "value" => 1, "date_add" => date("Y-m-d H:i:s"), "date_upd" => date("Y-m-d H:i:s")));
                if($insert){
                    $webservice = true;
                }
            }
        }

        if ( $webservice == true ){
            $tokenWS = Db::getInstance()->executeS("select * from {$this->dbPrefix}webservice_account where active = 1 and is_module = 1 order by id_webservice_account desc limit 1");
            if ( !empty($tokenWS) ) {
                $token = $tokenWS[0]["key"];
            } else {
                $token = $this->createToken();
            }
        }

        return $token;
    }

    protected function createToken()
    {
        $token = "ENVIA_";
        $token .= $this->generateRandomString(26);

        // CREATE WEBSERVICE ACCOUNT
        $account = Db::getInstance()->insert("webservice_account", array("key" => $token, "description" => "Created by Envia.com Prestashop Module", "class_name" => "WebserviceRequest", "is_module" => 1, "module_name" => $this->name, "active" => 1));
        $accountId = Db::getInstance()->Insert_ID();
        $shopId = Context::getContext()->shop->id;
        $accountShop = Db::getInstance()->insert("webservice_account_shop", array("id_webservice_account" => $accountId, "id_shop" => $shopId));
        $permissions = array(
            "GET" => array("addresses", "carriers", "cart_rules", "carts", "categories", "combinations", "configurations", "contacts", "content_management_system", "countries", "currencies", "customer_messages", "customer_threads", "customers", "customizations", "deliveries", "employees", "groups", "guests", "image_types", "images", "languages", "manufacturers", "messages", "order_carriers", "order_details", "order_histories", "order_invoices", "order_payments", "order_slip", "order_states", "orders", "price_ranges", "product_customization_fields", "product_feature_values", "product_features", "product_option_values", "product_options", "product_suppliers", "products", "search", "shop_groups", "shop_urls", "shops", "specific_price_rules", "specific_prices", "states", "stock_availables", "stock_movement_reasons", "stock_movements", "stocks", "stores", "suppliers", "supply_order_details", "supply_order_histories", "supply_order_receipt_histories", "supply_order_states", "supply_orders", "tags", "tax_rule_groups", "tax_rules", "taxes", "translated_configurations", "warehouse_product_locations", "warehouses", "weight_ranges", "zones"),
            "PUT" => array("addresses", "carriers", "cart_rules", "carts", "categories", "combinations", "configurations", "contacts", "content_management_system", "countries", "currencies", "customer_messages", "customer_threads", "customers", "customizations", "deliveries", "employees", "groups", "guests", "image_types", "images", "languages", "manufacturers", "messages", "order_carriers", "order_details", "order_histories", "order_invoices", "order_payments", "order_slip", "order_states", "orders", "price_ranges", "product_customization_fields", "product_feature_values", "product_features", "product_option_values", "product_options", "product_suppliers", "products", "shop_groups", "shop_urls", "shops", "specific_price_rules", "specific_prices", "states", "stock_availables", "stock_movement_reasons", "stores", "suppliers", "tags", "tax_rule_groups", "tax_rules", "taxes", "translated_configurations", "warehouses", "weight_ranges", "zones"),
            "POST" => array("addresses", "carriers", "cart_rules", "carts", "categories", "combinations", "configurations", "contacts", "content_management_system", "countries", "currencies", "customer_messages", "customer_threads", "customers", "customizations", "deliveries", "employees", "groups", "guests", "image_types", "images", "languages", "manufacturers", "messages", "order_carriers", "order_details", "order_histories", "order_invoices", "order_payments", "order_slip", "order_states", "orders", "price_ranges", "product_customization_fields", "product_feature_values", "product_features", "product_option_values", "product_options", "product_suppliers", "products", "shop_groups", "shop_urls", "shops", "specific_price_rules", "specific_prices", "states", "stock_movement_reasons", "stores", "suppliers", "tags", "tax_rule_groups", "tax_rules", "taxes", "translated_configurations", "warehouses", "weight_ranges", "zones"),
            "DELETE" => array("addresses", "carriers", "cart_rules", "carts", "categories", "combinations", "configurations", "contacts", "content_management_system", "countries", "currencies", "customer_messages", "customer_threads", "customers", "customizations", "deliveries", "employees", "groups", "guests", "image_types", "images", "languages", "manufacturers", "messages", "order_carriers", "order_details", "order_histories", "order_invoices", "order_payments", "order_slip", "order_states", "orders", "price_ranges", "product_customization_fields", "product_feature_values", "product_features", "product_option_values", "product_options", "product_suppliers", "products", "shop_groups", "shop_urls", "shops", "specific_price_rules", "specific_prices", "states", "stock_movement_reasons", "stores", "suppliers", "tags", "tax_rule_groups", "tax_rules", "taxes", "translated_configurations", "weight_ranges", "zones")
        );

        foreach ( $permissions as $key => $val ){
            foreach ( $val as $permission ){
                Db::getInstance()->insert("webservice_permission", array("resource" => $permission, "method" => $key, "id_webservice_account" => $accountId));
            }
        }

        return $token;
    }

    protected function generateRandomString($length = 10) 
    {
        $characters = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = Tools::strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    protected function uninstallApp()
    {
        Tools::file_get_contents("{$this->enviaHostname}/integrate/uninstall?shop={$this->url}&ecommerce=3");
        return true;
    }
}
